@@ -1,7 +1,7 @@
 """Plotting functions for manhattan plot.
 
 Copyright (c) Shujia Huang
-Date: 2016-01-23
+Date: 2021-02-21
 
 This model is based on brentp's script on github:
 https://github.com/brentp/bio-playground/blob/master/plots/manhattan-plot.py
@@ -15,6 +15,7 @@ import numpy as np
 
 import matplotlib.pyplot as plt
 from ._utils import General
+from ..utils import adjust_text
 
 
 # learn something from "https://github.com/reneshbedre/bioinfokit/blob/38fb4966827337f00421119a69259b92bb67a7d0/bioinfokit/visuz.py"
@@ -24,9 +25,8 @@ def manhattanplot(data, chrom="#CHROM", pos="POS", pv="P", snp="ID", logp=True,
                   xtick_label_set=None, CHR=None, xticklabel_kws=None,
                   suggestiveline=1e-5, genomewideline=5e-8, sign_line_cols="#D62728,#2CA02C", hline_kws=None,
                   sign_marker_p=None, sign_marker_color="r",
-                  is_annotate_topsnp=False, annotext_kws=None, ld_block_size=50000,
-                  is_show=True, dpi=300, figname=None, **kwargs):
-
+                  is_annotate_topsnp=False, text_kws=None, ld_block_size=50000,
+                  is_show=None, dpi=300, figname=None, **kwargs):
     """Creates a manhattan plot from PLINK assoc output (or any data frame with chromosome, position, and p-value).
 
     Parameters
@@ -114,14 +114,16 @@ def manhattanplot(data, chrom="#CHROM", pos="POS", pv="P", snp="ID", logp=True,
     is_annotate_topsnp : boolean, default is False, optional.
         Annotate the top SNP or not for the significant locus.
 
-    annotext_kws: key, value pairings, or None, optional
-        keyword arguments for plotting plt.annotate in`` matplotlib.pyplot.annotate(text, xy, *args, **kwargs)``
+    text_kws: key, value pairings, or None, optional
+        keyword arguments for plotting plt.annotate in`` matplotlib.axes.Axes.text(x, y, s, fontdict=None, **kwargs)``
 
     ld_block_size : integer, default is 50000, optional
         Set the size of LD block which for finding top SNP. And the top SNP's annotation represent the block.
 
-    is_show : boolean, default is True, optional
-        Display the plot or not.
+    is_show : boolean or None, default is None, Optional. 
+        Display the plot in screen or not.
+        You can set this parameter by your wish, or it'll set to be True automatically 
+        if ``is_show`` and ``figname`` are None simultaneously.
 
     dpi : float or 'figure', default is 300, optional.
         The resolution in dots-pet-inch for plot. If 'figure', use the figure's dpi value.
@@ -169,7 +171,7 @@ def manhattanplot(data, chrom="#CHROM", pos="POS", pv="P", snp="ID", logp=True,
     .. plot::
         :context: close-figs
 
-        >>> xtick = set(list(map(str, range(1, 15))) + ['16', '18', '20', '22', 'X'])
+        >>> xtick = set(['chr' + i for i in list(map(str, range(1, 10))) + ['11', '13', '15', '18', '21', 'X']])
         >>> manhattanplot(data=df, xlabel="Chromosome", ylabel=r"$-log_{10}{(P)}$",
         ...               xtick_label_set=xtick, figname="manhattan.png")
 
@@ -180,7 +182,7 @@ def manhattanplot(data, chrom="#CHROM", pos="POS", pv="P", snp="ID", logp=True,
         :context: close-figs
     
         >>> manhattanplot(data=df,
-        ...               hline_kws={"y": 3, "linestyle": "--", "lw": 1.3},
+        ...               hline_kws={"linestyle": "--", "lw": 1.3},
         ...               xlabel="Chromosome",
         ...               ylabel=r"$-log_{10}{(P)}$",
         ...               xtick_label_set = xtick)
@@ -191,7 +193,7 @@ def manhattanplot(data, chrom="#CHROM", pos="POS", pv="P", snp="ID", logp=True,
         :context: close-figs
 
         >>> manhattanplot(data=df,
-        ...               hline_kws={"y": 3, "linestyle": "--", "lw": 1.3},
+        ...               hline_kws={"linestyle": "--", "lw": 1.3},
         ...               xlabel="Chromosome",
         ...               ylabel=r"$-log_{10}{(P)}$",
         ...               xticklabel_kws={"rotation": "vertical"})
@@ -209,27 +211,20 @@ def manhattanplot(data, chrom="#CHROM", pos="POS", pv="P", snp="ID", logp=True,
         ...               sign_marker_color="r",
         ...               snp="ID",
         ...               title="Test",
-        ...               xtick_label_set=xtick,  # CHR='8', # specific showing the chromosome 8th
+        ...               xtick_label_set=xtick,
         ...               xlabel="Chromosome",
         ...               ylabel=r"$-log_{10}{(P)}$",
         ...               sign_line_cols=["#D62728", "#2CA02C"],
         ...               hline_kws={"linestyle": "--", "lw": 1.3},
         ...               is_annotate_topsnp=True,
         ...               ld_block_size=50000,  # 50000 bp
-        ...               annotext_kws={"size": 12,  # The fontsize of annotate text
-        ...                             "xycoords": "data",
-        ...                             "xytext": (15, +15),
-        ...                             "textcoords": "offset points",
-        ...                             "bbox": dict(boxstyle="round", alpha=0.2), 
-        ...                             "arrowprops": dict(arrowstyle="->",
-        ...                                                connectionstyle="angle,angleA=0,angleB=80,rad=10",
-        ...                                                alpha=0.6, relpos=(0, 0))},
+        ...               annotext_kws={"fontsize": 12,  # The fontsize of annotate text
+        ...                             "arrowprops": dict(arrowstyle="-", color="k", alpha=0.6)},
         ...               dpi=300,  # set the resolution of plot figure
         ...               is_show=False,  # do not show the figure
         ...               figname="output_manhattan_plot.png",
         ...               ax=ax)
     """
-
     if not isinstance(data, DataFrame):
         raise ValueError("[ERROR] Input data must be a pandas.DataFrame.")
     if chrom not in data:
@@ -255,8 +250,8 @@ def manhattanplot(data, chrom="#CHROM", pos="POS", pv="P", snp="ID", logp=True,
         xticklabel_kws = {}
     if hline_kws is None:
         hline_kws = {}
-    if annotext_kws is None:
-        annotext_kws = {}
+    if text_kws is None:
+        text_kws = {}
 
     if "," in color:
         color = color.split(",")
@@ -316,8 +311,8 @@ def manhattanplot(data, chrom="#CHROM", pos="POS", pv="P", snp="ID", logp=True,
     # Plotting the Top SNP for each significant block
     if is_annotate_topsnp:
         sign_top_snp = _find_top_snp(sign_snp_sites, ld_block_size=ld_block_size, is_get_biggest=logp)
-        for _x, _y, _text in sign_top_snp:
-            ax.annotate(_text, xy=(_x, _y), **annotext_kws)
+        texts = [ax.text(_x, _y, _text) for _x, _y, _text in sign_top_snp]
+        adjust_text(texts, ax=ax, **text_kws)
 
     if CHR is None:
 
@@ -346,6 +341,9 @@ def manhattanplot(data, chrom="#CHROM", pos="POS", pv="P", snp="ID", logp=True,
     ax.spines["top"].set_visible(False)
     ax.spines["right"].set_visible(False)
 
+    if (is_show is None) and (figname is None):
+        is_show = True
+
     General.get_figure(is_show, fig_name=figname, dpi=dpi)
     return ax
 
